/*
 * Copyright (c) 1996, 1997, 1998 Shigio Yamaguchi. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Shigio Yamaguchi.
 * 4. Neither the name of the author nor the names of any co-contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	find.c					1-May-98
 *
 */
#include <sys/param.h>

#include <assert.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <regex.h>

#include "conf.h"
#include "die.h"
#include "find.h"
#include "gparam.h"
#include "locatestring.h"
#include "strbuf.h"

/*
 * usage of findxxx()
 *
 *	findopen(db);
 *	while (path = findread(&length)) {
 *		...
 *	}
 *	findclose();
 *
 */
static regex_t	skip_area;
static regex_t	*skip = &skip_area;
static FILE	*ip;
static int	opened;

static void	trim __P((char *));

/*
 * trim: remove blanks and '\'.
 */
static void
trim(s)
char	*s;
{
	char	*p;

	for (p = s; *s; s++) {
		if (isspace(*s))
			continue;	
		if (*s == '\\' && *(s + 1))
			s++;
		*p++ = *s;
	}
	*p = 0;
}
void
findopen()
{
	char	*findcom, *p, *q;
	STRBUF	*sb;
	char	*sufflist = NULL;
	char	*skiplist = NULL;

	assert(opened == 0);
	opened = 1;

	sb = stropen();
	if (!getconfs("suffixes", sb))
		die("cannot get suffixes data.");
	sufflist = strdup(strget(sb));
	if (!sufflist)
		die("short of memory.");
	trim(sufflist);
	strstart(sb);
	if (getconfs("skip", sb)) {
		skiplist = strdup(strget(sb));
		if (!skiplist)
			die("short of memory.");
		trim(skiplist);
	}

	strstart(sb);
	strputs(sb, "find . \\( -type f -o -type l \\) \\(");
	for (p = sufflist; p; ) {
		char	*suff = p;
		if ((p = locatestring(p, ",", MATCH_FIRST)) != NULL)
			*p++ = 0;
		strputs(sb, " -name '*.");
		strputs(sb, suff);
		strputs(sb, "'");
		if (p)
			strputs(sb, " -o");
	}
	strputs(sb, " \\) -print");
	findcom = strget(sb);

	if (skiplist) {
		char	*reg;
		STRBUF	*sbb = stropen();
		/*
		 * construct regular expression.
		 */
		strputc(sbb, '(');	/* ) */
		for (p = skiplist; p; ) {
			char    *skipf = p;
			if ((p = locatestring(p, ",", MATCH_FIRST)) != NULL)
				*p++ = 0;
			strputc(sbb, '/');
			for (q = skipf; *q; q++) {
				if (*q == '.')
					strputc(sbb, '\\');
				strputc(sbb, *q);
			}
			if (*(q - 1) != '/')
				strputc(sbb, '$');
			if (p)
				strputc(sbb, '|');
		}
		strputc(sbb, ')');
		reg = strget(sbb);
		/*
		 * compile regular expression.
		 */
		if (regcomp(skip, reg, REG_EXTENDED|REG_NEWLINE) != 0)
			die("cannot compile regular expression.");
		strclose(sbb);
	} else {
		skip = (regex_t *)0;
	}
	if (!(ip = popen(findcom, "r")))
		die("cannot execute find.");
	strclose(sb);
	if (sufflist)
		free(sufflist);
	if (skiplist)
		free(skiplist);
}
char	*
findread(length)
int	*length;
{
	static char	path[MAXPATHLEN+1];
	char	*p;

	assert(opened == 1);
	while (fgets(path, MAXPATHLEN, ip)) {
		if (!skip || regexec(skip, path, 0, 0, 0) != 0) {
			/*
			 * chop(path)
			 */
			p = path + strlen(path) - 1;
			if (*p != '\n')
				die("output of find(1) is wrong (findread).");
			*p = 0;
			if (length)
				*length = p - path;
			return path;
		}
	}
	return NULL;
}
void
findclose(void)
{
	assert(opened == 1);
	pclose(ip);
	opened = 0;
}
